@@ -5,6 +5,7 @@ import traceback
 import zipfile
 
 from extract_msg import __doc__, utils
+from extract_msg.enums import AttachErrorBehavior
 
 
 def main() -> None:
@@ -66,7 +67,9 @@ def main() -> None:
             'pdf': args.pdf,
             'preparedHtml': args.preparedHtml,
             'rtf': args.rtf,
+            'skipBodyNotFound': args.skipBodyNotFound,
             'skipEmbedded': args.skipEmbedded,
+            'skipNotImplemented': args.skipNotImplemented,
             'useMsgFilename': args.useFilename,
             'wkOptions': args.wkOptions,
             'wkPath': args.wkPath,
@@ -77,6 +80,11 @@ def main() -> None:
             'ignoreRtfDeErrors': args.ignoreRtfDeErrors,
         }
 
+        # If we are skipping the NotImplementedError attachments, we need to
+        # suppress the error.
+        if args.skipNotImplemented:
+            openKwargs['attachmentErrorBehavior'] = AttachErrorBehavior.NOT_IMPLEMENTED
+
         def strSanitize(inp):
             """
             Small function to santize parts of a string when failing to print
